@@ -1,7 +1,7 @@
 /**************************************************************************
  *   help.c  --  This file is part of GNU nano.                           *
  *                                                                        *
- *   Copyright (C) 2000-2011, 2013-2021 Free Software Foundation, Inc.    *
+ *   Copyright (C) 2000-2011, 2013-2022 Free Software Foundation, Inc.    *
  *   Copyright (C) 2017 Rishabh Dave                                      *
  *   Copyright (C) 2014-2019 Benno Schulenberg                            *
  *                                                                        *
@@ -463,7 +463,7 @@ void show_help(void)
 		focusing = TRUE;
 
 		/* Show the cursor when we searched and found something. */
-		kbinput = get_kbinput(edit, didfind == 1 || ISSET(SHOW_CURSOR));
+		kbinput = get_kbinput(midwin, didfind == 1 || ISSET(SHOW_CURSOR));
 
 		didfind = 0;
 
