@@ -1,12 +1,12 @@
 /**************************************************************************
  *   text.c  --  This file is part of GNU nano.                           *
  *                                                                        *
- *   Copyright (C) 1999-2011, 2013-2021 Free Software Foundation, Inc.    *
+ *   Copyright (C) 1999-2011, 2013-2022 Free Software Foundation, Inc.    *
  *   Copyright (C) 2014-2015 Mark Majeres                                 *
  *   Copyright (C) 2016 Mike Scalora                                      *
  *   Copyright (C) 2016 Sumedh Pendurkar                                  *
  *   Copyright (C) 2018 Marco Diego Aurélio Mesquita                      *
- *   Copyright (C) 2015-2020 Benno Schulenberg                            *
+ *   Copyright (C) 2015-2021 Benno Schulenberg                            *
  *                                                                        *
  *   GNU nano is free software: you can redistribute it and/or modify     *
  *   it under the terms of the GNU General Public License as published    *
@@ -39,7 +39,7 @@
 #ifdef ENABLE_WORDCOMPLETION
 static int pletion_x = 0;
 		/* The x position in pletion_line of the last found completion. */
-static completion_word *list_of_completions;
+static completionstruct *list_of_completions;
 		/* A linked list of the completions that have been attempted. */
 #endif
 
@@ -58,10 +58,10 @@ void do_mark(void)
 		refresh_needed = TRUE;
 	}
 }
-#endif /* !NANO_TINY */
+#endif
 
-/* Insert a tab.  If the TABS_TO_SPACES flag is set, insert the number
- * of spaces that a tab would normally take up. */
+/* Insert a tab.  Or, if --tabstospaces is in effect, insert the number
+ * of spaces that a tab would normally take up at this position. */
 void do_tab(void)
 {
 #ifdef ENABLE_COLOR
@@ -133,12 +133,12 @@ void do_indent(void)
 
 	indentation = nmalloc(tabsize + 1);
 
-	/* Set the indentation to either a bunch of spaces or a single tab. */
 #ifdef ENABLE_COLOR
 	if (openfile->syntax && openfile->syntax->tab)
 		indentation = mallocstrcpy(indentation, openfile->syntax->tab);
 	else
 #endif
+	/* Set the indentation to either a bunch of spaces or a single tab. */
 	if (ISSET(TABS_TO_SPACES)) {
 		memset(indentation, ' ', tabsize);
 		indentation[tabsize] = '\0';
@@ -186,7 +186,7 @@ size_t length_of_white(const char *text)
 
 	while (TRUE) {
 		if (*text == '\t')
-			return ++white_count;
+			return white_count + 1;
 
 		if (*text != ' ')
 			return white_count;
@@ -278,9 +278,6 @@ void handle_indent_action(undostruct *u, bool undoing, bool add_indent)
 	groupstruct *group = u->grouping;
 	linestruct *line = line_from_number(group->top_line);
 
-	if (group->next != NULL)
-		die("Multiple groups -- please report a bug\n");
-
 	/* When redoing, reposition the cursor and let the indenter adjust it. */
 	if (!undoing)
 		goto_line_posx(u->head_lineno, u->head_x);
@@ -418,11 +415,9 @@ void do_comment(void)
 
 	/* Comment/uncomment each of the selected lines when possible, and
 	 * store undo data when a line changed. */
-	for (line = top; line != bot->next; line = line->next) {
-		if (comment_line(action, line, comment_seq)) {
+	for (line = top; line != bot->next; line = line->next)
+		if (comment_line(action, line, comment_seq))
 			update_multiline_undo(line->lineno, "");
-		}
-	}
 
 	set_modified();
 	ensure_firstcolumn_is_aligned();
@@ -468,6 +463,10 @@ void undo_cut(undostruct *u)
 {
 	goto_line_posx(u->head_lineno, (u->xflags & WAS_WHOLE_LINE) ? 0 : u->head_x);
 
+	/* Clear an inherited anchor but not a user-placed one. */
+	if (!(u->xflags & HAD_ANCHOR_AT_START))
+		openfile->current->has_anchor = FALSE;
+
 	if (u->cutbuffer)
 		copy_from_buffer(u->cutbuffer);
 
@@ -666,7 +665,7 @@ void do_undo(void)
 
 	openfile->totsize = u->wassize;
 
-	/* When at the point where the file was last saved, unset "Modified". */
+	/* When at the point where the buffer was last saved, unset "Modified". */
 	if (openfile->current_undo == openfile->last_saved) {
 		openfile->modified = FALSE;
 		titlebar(NULL);
@@ -826,7 +825,7 @@ void do_redo(void)
 
 	openfile->totsize = u->newsize;
 
-	/* When at the point where the file was last saved, unset "Modified". */
+	/* When at the point where the buffer was last saved, unset "Modified". */
 	if (openfile->current_undo == openfile->last_saved) {
 		openfile->modified = FALSE;
 		titlebar(NULL);
@@ -1028,6 +1027,8 @@ void add_undo(undo_type action, const char *message)
 #endif
 	case CUT_TO_EOF:
 		u->xflags |= (INCLUDED_LAST_LINE | CURSOR_WAS_AT_HEAD);
+		if (openfile->current->has_anchor)
+			u->xflags |= HAD_ANCHOR_AT_START;
 		break;
 	case ZAP:
 	case CUT:
@@ -1049,6 +1050,9 @@ void add_undo(undo_type action, const char *message)
 			u->tail_x = 0;
 		} else
 			u->xflags |= CURSOR_WAS_AT_HEAD;
+		if ((openfile->mark && mark_is_before_cursor() && openfile->mark->has_anchor) ||
+				((!openfile->mark || !mark_is_before_cursor()) && openfile->current->has_anchor))
+			u->xflags |= HAD_ANCHOR_AT_START;
 		break;
 	case PASTE:
 		u->cutbuffer = copy_buffer(cutbuffer);
@@ -1087,24 +1091,24 @@ void update_multiline_undo(ssize_t lineno, char *indentation)
 	/* If there already is a group and the current line is contiguous with it,
 	 * extend the group; otherwise, create a new group. */
 	if (u->grouping && u->grouping->bottom_line + 1 == lineno) {
-		size_t number_of_lines;
+		size_t number_of_lines = lineno - u->grouping->top_line + 1;
 
-		u->grouping->bottom_line++;
+		u->grouping->bottom_line = lineno;
 
-		number_of_lines = u->grouping->bottom_line - u->grouping->top_line + 1;
 		u->grouping->indentations = nrealloc(u->grouping->indentations,
 										number_of_lines * sizeof(char *));
 		u->grouping->indentations[number_of_lines - 1] = copy_of(indentation);
 	} else {
 		groupstruct *born = nmalloc(sizeof(groupstruct));
 
-		born->next = u->grouping;
-		u->grouping = born;
 		born->top_line = lineno;
 		born->bottom_line = lineno;
 
-		u->grouping->indentations = nmalloc(sizeof(char *));
-		u->grouping->indentations[0] = copy_of(indentation);
+		born->indentations = nmalloc(sizeof(char *));
+		born->indentations[0] = copy_of(indentation);
+
+		born->next = u->grouping;
+		u->grouping = born;
 	}
 
 	/* Store the file size after the change, to be used when redoing. */
@@ -1208,9 +1212,9 @@ void update_undo(undo_type action)
 
 #ifdef ENABLE_WRAPPING
 /* When the current line is overlong, hard-wrap it at the furthest possible
- * whitespace character, and (if possible) prepend the remainder of the line
- * to the next line.  Return TRUE if wrapping occurred, and FALSE otherwise. */
-bool do_wrap(void)
+ * whitespace character, and prepend the excess part to an "overflow" line
+ * (when it already exists, otherwise create one). */
+void do_wrap(void)
 {
 	linestruct *line = openfile->current;
 		/* The line to be wrapped, if needed and possible. */
@@ -1239,7 +1243,7 @@ bool do_wrap(void)
 
 	/* If no wrapping point was found before end-of-line, we don't wrap. */
 	if (wrap_loc < 0 || lead_len + wrap_loc == line_len)
-		return FALSE;
+		return;
 
 	/* Adjust the wrap location to its position in the full line,
 	 * and step forward to the character just after the blank. */
@@ -1247,7 +1251,7 @@ bool do_wrap(void)
 
 	/* When now at end-of-line, no need to wrap. */
 	if (line->data[wrap_loc] == '\0')
-		return FALSE;
+		return;
 
 #ifndef NANO_TINY
 	add_undo(SPLIT_BEGIN, NULL);
@@ -1332,7 +1336,9 @@ bool do_wrap(void)
 		strncpy(line->data, line->prev->data, lead_len);
 
 		openfile->current_x += lead_len;
+		openfile->totsize += lead_len;
 #ifndef NANO_TINY
+		free(openfile->undotop->strdata);
 		update_undo(ENTER);
 #endif
 		if (autowhite)
@@ -1354,7 +1360,7 @@ bool do_wrap(void)
 	add_undo(SPLIT_END, NULL);
 #endif
 
-	return TRUE;
+	refresh_needed = TRUE;
 }
 #endif /* ENABLE_WRAPPING */
 
@@ -1459,9 +1465,8 @@ bool begpar(const linestruct *const line, int depth)
 {
 	size_t quot_len, indent_len, prev_dent_len;
 
-	/* If this is the very first line of the buffer, it counts as a BOP
-	 * even when it contains no text. */
-	if (line == openfile->filetop)
+	/* The very first line counts as a BOP, even when it contains no text. */
+	if (line->prev == NULL)
 		return TRUE;
 
 	/* If recursion is going too deep, just say it's not a BOP. */
@@ -1707,9 +1712,12 @@ void justify_paragraph(linestruct **line, size_t count)
 	free(lead_string);
 }
 
-/* Justify the current paragraph, or the entire buffer when full_justify is
+#define ONE_PARAGRAPH  FALSE
+#define WHOLE_BUFFER  TRUE
+
+/* Justify the current paragraph, or the entire buffer when whole_buffer is
  * TRUE.  But if the mark is on, justify only the marked text instead. */
-void do_justify(bool full_justify)
+void justify_text(bool whole_buffer)
 {
 	size_t linecount;
 		/* The number of lines in the original paragraph. */
@@ -1825,7 +1833,7 @@ void do_justify(bool full_justify)
 	{
 		/* When justifying the entire buffer, start at the top.  Otherwise, when
 		 * in a paragraph but not at its beginning, move back to its first line. */
-		if (full_justify)
+		if (whole_buffer)
 			openfile->current = openfile->filetop;
 		else if (inpar(openfile->current) && !begpar(openfile->current, 0))
 			do_para_begin(&openfile->current);
@@ -1847,7 +1855,7 @@ void do_justify(bool full_justify)
 		start_x = 0;
 
 		/* Set the end point of the paragraph. */
-		if (full_justify)
+		if (whole_buffer)
 			endline = openfile->filebot;
 		else {
 			endline = startline;
@@ -1922,7 +1930,7 @@ void do_justify(bool full_justify)
 		justify_paragraph(&jusline, linecount);
 
 		/* When justifying the entire buffer, find and justify all paragraphs. */
-		if (full_justify) {
+		if (whole_buffer) {
 			while (find_paragraph(&jusline, &linecount)) {
 				justify_paragraph(&jusline, linecount);
 
@@ -1933,9 +1941,11 @@ void do_justify(bool full_justify)
 	}
 
 #ifndef NANO_TINY
-	add_undo(PASTE, NULL);
-	if (full_justify && !openfile->mark && !cutbuffer->has_anchor)
+	/* Wipe an anchor on the first paragraph if it was only inherited. */
+	if (whole_buffer && !openfile->mark && !cutbuffer->has_anchor)
 		openfile->current->has_anchor = FALSE;
+
+	add_undo(PASTE, NULL);
 #endif
 	/* Do the equivalent of a paste of the justified text. */
 	ingraft_buffer(cutbuffer);
@@ -1954,22 +1964,20 @@ void do_justify(bool full_justify)
 	}
 
 	add_undo(COUPLE_END, N_("justification"));
-#endif
 
-	/* We're done justifying.  Restore the old cutbuffer. */
-	cutbuffer = was_cutbuffer;
-
-	/* Show what we justified on the status bar. */
-#ifndef NANO_TINY
+	/* Report on the status bar what we justified. */
 	if (openfile->mark)
 		statusline(REMARK, _("Justified selection"));
 	else
 #endif
-	if (full_justify)
+	if (whole_buffer)
 		statusline(REMARK, _("Justified file"));
 	else
 		statusbar(_("Justified paragraph"));
 
+	/* We're done justifying.  Restore the cutbuffer. */
+	cutbuffer = was_cutbuffer;
+
 	/* Set the desired screen column (always zero, except at EOF). */
 	openfile->placewewant = xplustabs();
 
@@ -1979,15 +1987,15 @@ void do_justify(bool full_justify)
 }
 
 /* Justify the current paragraph. */
-void do_justify_void(void)
+void do_justify(void)
 {
-	do_justify(FALSE);
+	justify_text(ONE_PARAGRAPH);
 }
 
 /* Justify the entire file. */
 void do_full_justify(void)
 {
-	do_justify(TRUE);
+	justify_text(WHOLE_BUFFER);
 	ran_a_tool = TRUE;
 }
 #endif /* ENABLE_JUSTIFY */
@@ -2052,9 +2060,9 @@ bool replace_buffer(const char *filename, undo_type action, const char *operatio
 /* Execute the given program, with the given temp file as last argument. */
 void treat(char *tempfile_name, char *theprogram, bool spelling)
 {
-	ssize_t lineno_save = openfile->current->lineno;
-	size_t current_x_save = openfile->current_x;
-	size_t pww_save = openfile->placewewant;
+	ssize_t was_lineno = openfile->current->lineno;
+	size_t was_pww = openfile->placewewant;
+	size_t was_x = openfile->current_x;
 	bool was_at_eol = (openfile->current->data[openfile->current_x] == '\0');
 	struct stat fileinfo;
 	long timestamp_sec = 0;
@@ -2138,7 +2146,7 @@ void treat(char *tempfile_name, char *theprogram, bool spelling)
 		/* Adjust the end point of the marked region for any change in
 		 * length of the region's last line. */
 		if (upright)
-			current_x_save = openfile->current_x;
+			was_x = openfile->current_x;
 		else
 			openfile->mark_x = openfile->current_x;
 
@@ -2156,7 +2164,7 @@ void treat(char *tempfile_name, char *theprogram, bool spelling)
 	}
 
 	/* Go back to the old position. */
-	goto_line_posx(lineno_save, current_x_save);
+	goto_line_posx(was_lineno, was_x);
 	if (was_at_eol || openfile->current_x > strlen(openfile->current->data))
 		openfile->current_x = strlen(openfile->current->data);
 
@@ -2167,7 +2175,7 @@ void treat(char *tempfile_name, char *theprogram, bool spelling)
 #endif
 	}
 
-	openfile->placewewant = pww_save;
+	openfile->placewewant = was_pww;
 	adjust_viewport(STATIONARY);
 
 	if (spelling)
@@ -2490,10 +2498,10 @@ void do_spell(void)
 
 #ifndef NANO_TINY
 	if (openfile->mark)
-		okay = write_marked_file(temp_name, stream, TRUE, OVERWRITE);
+		okay = write_region_to_file(temp_name, stream, TEMPORARY, OVERWRITE);
 	else
 #endif
-		okay = write_file(temp_name, stream, TRUE, OVERWRITE, TRUE);
+		okay = write_file(temp_name, stream, TEMPORARY, OVERWRITE, NONOTES);
 
 	if (!okay) {
 		statusline(ALERT, _("Error writing temp file: %s"), strerror(errno));
@@ -2550,12 +2558,12 @@ void do_linter(void)
 	edit_refresh();
 
 	if (openfile->modified) {
-		int choice = do_yesno_prompt(FALSE, _("Save modified buffer before linting?"));
+		int choice = ask_user(YESORNO, _("Save modified buffer before linting?"));
 
-		if (choice == -1) {
+		if (choice == CANCEL) {
 			statusbar(_("Cancelled"));
 			return;
-		} else if (choice == 1 && (do_writeout(FALSE, FALSE) != 1))
+		} else if (choice == YES && (write_it_out(FALSE, FALSE) != 1))
 			return;
 	}
 
@@ -2610,6 +2618,9 @@ void do_linter(void)
 		return;
 	}
 
+	/* Block resizing signals while reading from the pipe. */
+	block_sigwinch(TRUE);
+
 	/* Read in the returned syntax errors. */
 	totalread = 0;
 	buffersize = pipesize + 1;
@@ -2626,47 +2637,41 @@ void do_linter(void)
 	*pointer = '\0';
 	close(lint_fd[0]);
 
-	/* Process the linter output. */
+	block_sigwinch(FALSE);
+
 	pointer = lintings;
 	onelint = lintings;
 
+	/* Now parse the output of the linter. */
 	while (*pointer != '\0') {
 		if ((*pointer == '\r') || (*pointer == '\n')) {
 			*pointer = '\0';
 			if (onelint != pointer) {
-				char *filename = NULL, *linestr = NULL, *maybecol = NULL;
-				char *message = copy_of(onelint);
+				char *filename, *linestring, *colstring;
+				char *complaint = copy_of(onelint);
+				char *spacer = strstr(complaint, " ");
 
-				/* At the moment we handle the following formats:
-				 *
-				 * filenameorcategory:line:column:message (e.g. splint)
-				 * filenameorcategory:line,column:message (e.g. pylint)
-				 * filenameorcategory:line:message        (e.g. pyflakes) */
-				if (strstr(message, ": ") != NULL) {
-					filename = strtok(onelint, ":");
-					if ((linestr = strtok(NULL, ":")) != NULL) {
-						if ((maybecol = strtok(NULL, ":")) != NULL) {
-							ssize_t tmplineno = 0, tmpcolno = 0;
-							char *tmplinecol;
+				/* The recognized format is "filename:line:column: message",
+				 * where ":column" may be absent or be ",column" instead. */
+				if ((filename = strtok(onelint, ":")) && spacer) {
+					if ((linestring = strtok(NULL, ":"))) {
+						if ((colstring = strtok(NULL, " "))) {
+							ssize_t linenumber = strtol(linestring, NULL, 10);
+							ssize_t colnumber = strtol(colstring, NULL, 10);
 
-							tmplineno = strtol(linestr, NULL, 10);
-							if (tmplineno <= 0) {
+							if (linenumber <= 0) {
+								free(complaint);
 								pointer++;
-								free(message);
 								continue;
 							}
 
-							tmpcolno = strtol(maybecol, NULL, 10);
-							/* Check if the middle field is in comma format. */
-							if (tmpcolno <= 0) {
-								strtok(linestr, ",");
-								if ((tmplinecol = strtok(NULL, ",")) != NULL)
-									tmpcolno = strtol(tmplinecol, NULL, 10);
-								else
-									tmpcolno = 1;
+							if (colnumber <= 0) {
+								colnumber = 1;
+								strtok(linestring, ",");
+								if ((colstring = strtok(NULL, ",")))
+									colnumber = strtol(colstring, NULL, 10);
 							}
 
-							/* Nice.  We have a lint message we can use. */
 							parsesuccess = TRUE;
 							tmplint = curlint;
 							curlint = nmalloc(sizeof(lintstruct));
@@ -2674,17 +2679,17 @@ void do_linter(void)
 							curlint->prev = tmplint;
 							if (curlint->prev != NULL)
 								curlint->prev->next = curlint;
-							curlint->msg = copy_of(message);
-							curlint->lineno = tmplineno;
-							curlint->colno = tmpcolno;
 							curlint->filename = copy_of(filename);
+							curlint->lineno = linenumber;
+							curlint->colno = colnumber;
+							curlint->msg = copy_of(spacer + 1);
 
 							if (lints == NULL)
 								lints = curlint;
 						}
 					}
 				}
-				free(message);
+				free(complaint);
 			}
 			onelint = pointer + 1;
 		}
@@ -2707,7 +2712,7 @@ void do_linter(void)
 		return;
 	}
 
-	if (helpless && LINES > 4) {
+	if (helpless && LINES > 5) {
 		UNSET(NO_HELP);
 		window_init();
 	}
@@ -2742,14 +2747,14 @@ void do_linter(void)
 
 				sprintf(msg, _("This message is for unopened file %s,"
 							" open it in a new buffer?"), curlint->filename);
-				choice = do_yesno_prompt(FALSE, msg);
+				choice = ask_user(YESORNO, msg);
 				currmenu = MLINTER;
 				free(msg);
 
-				if (choice == -1) {
+				if (choice == CANCEL) {
 					statusbar(_("Cancelled"));
 					break;
-				} else if (choice == 1) {
+				} else if (choice == YES) {
 					open_buffer(curlint->filename, TRUE);
 				} else {
 #endif
@@ -2808,9 +2813,9 @@ void do_linter(void)
 
 		/* Place the cursor to indicate the affected line. */
 		place_the_cursor();
-		wnoutrefresh(edit);
+		wnoutrefresh(midwin);
 
-		kbinput = get_kbinput(bottomwin, VISIBLE);
+		kbinput = get_kbinput(footwin, VISIBLE);
 
 #ifndef NANO_TINY
 		if (kbinput == KEY_WINCH)
@@ -2892,7 +2897,7 @@ void do_formatter(void)
 	temp_name = safe_tempfile(&stream);
 
 	if (temp_name != NULL)
-		okay = write_file(temp_name, stream, TRUE, OVERWRITE, TRUE);
+		okay = write_file(temp_name, stream, TEMPORARY, OVERWRITE, NONOTES);
 
 	if (!okay) {
 		statusline(ALERT, _("Error writing temp file: %s"), strerror(errno));
@@ -2908,9 +2913,9 @@ void do_formatter(void)
 #endif /* ENABLE_COLOR */
 
 #ifndef NANO_TINY
-/* Our own version of "wc".  Note that its character counts are in
+/* Our own version of "wc".  Note that the character count is in
  * multibyte characters instead of single-byte characters. */
-void do_wordlinechar_count(void)
+void count_lines_words_and_characters(void)
 {
 	linestruct *was_current = openfile->current;
 	size_t was_x = openfile->current_x;
@@ -2949,7 +2954,7 @@ void do_wordlinechar_count(void)
 	 * incrementing the word count for each successful step. */
 	while (openfile->current->lineno < botline->lineno ||
 				(openfile->current == botline && openfile->current_x < bot_x)) {
-		if (do_next_word(FALSE, TRUE))
+		if (do_next_word(FALSE))
 			words++;
 	}
 
@@ -2977,7 +2982,7 @@ void do_verbatim_input(void)
 	place_the_cursor();
 
 	/* Read in the first one or two bytes of the next keystroke. */
-	bytes = get_verbatim_kbinput(edit, &count);
+	bytes = get_verbatim_kbinput(midwin, &count);
 
 	/* When something valid was obtained, unsuppress cursor-position display,
 	 * insert the bytes into the edit buffer, and blank the status bar. */
@@ -3025,7 +3030,7 @@ void complete_a_word(void)
 	char *shard, *completion = NULL;
 	size_t start_of_shard, shard_length = 0;
 	size_t i = 0, j = 0;
-	completion_word *some_word;
+	completionstruct *some_word;
 #ifdef ENABLE_WRAPPING
 	bool was_set_wrapping = ISSET(BREAK_LONG_LINES);
 #endif
@@ -3034,7 +3039,7 @@ void complete_a_word(void)
 	if (pletion_line == NULL) {
 		/* Clear the list of words of a previous completion run. */
 		while (list_of_completions != NULL) {
-			completion_word *dropit = list_of_completions;
+			completionstruct *dropit = list_of_completions;
 			list_of_completions = list_of_completions->next;
 			free(dropit->word);
 			free(dropit);
@@ -3127,7 +3132,7 @@ void complete_a_word(void)
 			}
 
 			/* Add the found word to the list of completions. */
-			some_word = nmalloc(sizeof(completion_word));
+			some_word = nmalloc(sizeof(completionstruct));
 			some_word->word = completion;
 			some_word->next = list_of_completions;
 			list_of_completions = some_word;
@@ -3159,8 +3164,8 @@ void complete_a_word(void)
 
 	/* The search has reached the end of the file. */
 	if (list_of_completions != NULL) {
+		edit_refresh();
 		statusline(AHEM, _("No further matches"));
-		refresh_needed = TRUE;
 	} else
 		/* TRANSLATORS: Shown when there are zero possible completions. */
 		statusline(AHEM, _("No matches"));
