@@ -1,7 +1,7 @@
 /**************************************************************************
  *   utils.c  --  This file is part of GNU nano.                          *
  *                                                                        *
- *   Copyright (C) 1999-2011, 2013-2021 Free Software Foundation, Inc.    *
+ *   Copyright (C) 1999-2011, 2013-2022 Free Software Foundation, Inc.    *
  *   Copyright (C) 2016, 2017, 2019 Benno Schulenberg                     *
  *                                                                        *
  *   GNU nano is free software: you can redistribute it and/or modify     *
@@ -61,7 +61,7 @@ const char *tail(const char *path)
 	if (slash == NULL)
 		return path;
 	else
-		return ++slash;
+		return slash + 1;
 }
 
 /* Return a copy of the two given strings, welded together. */
@@ -282,34 +282,26 @@ const char *strstrwrapper(const char *haystack, const char *needle,
 		return mbstrcasestr(start, needle);
 }
 
-/* This is a wrapper for the malloc() function that properly handles
- * things when we run out of memory. */
+/* Allocate the given amount of memory and return a pointer to it. */
 void *nmalloc(size_t howmuch)
 {
-	void *r = malloc(howmuch);
+	void *section = malloc(howmuch);
 
-	if (howmuch == 0)
-		die("Allocating zero bytes.  Please report a bug.\n");
-
-	if (r == NULL)
+	if (section == NULL)
 		die(_("Nano is out of memory!\n"));
 
-	return r;
+	return section;
 }
 
-/* This is a wrapper for the realloc() function that properly handles
- * things when we run out of memory. */
-void *nrealloc(void *ptr, size_t howmuch)
+/* Reallocate the given section of memory to have the given size. */
+void *nrealloc(void *section, size_t howmuch)
 {
-	void *r = realloc(ptr, howmuch);
+	section = realloc(section, howmuch);
 
-	if (howmuch == 0)
-		die("Allocating zero bytes.  Please report a bug.\n");
-
-	if (r == NULL)
+	if (section == NULL)
 		die(_("Nano is out of memory!\n"));
 
-	return r;
+	return section;
 }
 
 /* Return an appropriately reallocated dest string holding a copy of src.
